@@ -80,6 +80,30 @@ typedef struct {
 	GtkTreeIter           *iter;
 } FilterResultCallbackData;
 
+static FilterResultCallbackData *
+filter_result_callback_data_new (EmpathyAccountChooser *chooser,
+				 TpAccount             *account,
+				 GtkTreeIter           *iter)
+{
+	FilterResultCallbackData *data;
+
+	data = g_slice_new0 (FilterResultCallbackData);
+	data->chooser = g_object_ref (chooser);
+	data->account = g_object_ref (account);
+	data->iter = gtk_tree_iter_copy (iter);
+
+	return data;
+}
+
+static void
+filter_result_callback_data_free (FilterResultCallbackData *data)
+{
+	g_object_unref (data->chooser);
+	g_object_unref (data->account);
+	gtk_tree_iter_free (data->iter);
+	g_slice_free (FilterResultCallbackData, data);
+}
+
 /* Distinguishes between store entries which are actually accounts, and special
  * items like the "All" entry and the separator below it, so they can be sorted
  * correctly. Higher-numbered entries will sort earlier.
@@ -99,6 +123,7 @@ enum {
 	COL_ACCOUNT_COUNT
 };
 
+static void     account_chooser_constructed            (GObject                  *object);
 static void     account_chooser_finalize               (GObject                  *object);
 static void     account_chooser_get_property           (GObject                  *object,
 							guint                     param_id,
@@ -156,6 +181,7 @@ empathy_account_chooser_class_init (EmpathyAccountChooserClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->constructed = account_chooser_constructed;
 	object_class->finalize = account_chooser_finalize;
 	object_class->get_property = account_chooser_get_property;
 	object_class->set_property = account_chooser_set_property;
@@ -206,8 +232,14 @@ empathy_account_chooser_init (EmpathyAccountChooser *chooser)
 	g_signal_connect (priv->manager, "account-removed",
 			  G_CALLBACK (account_chooser_account_removed_cb),
 			  chooser);
+}
 
-	account_chooser_setup (EMPATHY_ACCOUNT_CHOOSER (chooser));
+static void
+account_chooser_constructed (GObject *object)
+{
+	EmpathyAccountChooser *self = (EmpathyAccountChooser *) object;
+
+	account_chooser_setup (self);
 }
 
 static void
@@ -284,30 +316,6 @@ empathy_account_chooser_new (void)
 	return chooser;
 }
 
-gboolean
-empathy_account_chooser_has_all_selected (EmpathyAccountChooser *chooser)
-{
-	EmpathyAccountChooserPriv *priv;
-	GtkTreeModel              *model;
-	GtkTreeIter                iter;
-	RowType                    type;
-
-	g_return_val_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser), FALSE);
-
-	priv = GET_PRIV (chooser);
-
-	g_return_val_if_fail (priv->has_all_option == TRUE, FALSE);
-
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (chooser));
-	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (chooser), &iter)) {
-		return FALSE;
-	}
-
-	gtk_tree_model_get (model, &iter, COL_ACCOUNT_ROW_TYPE, &type, -1);
-
-	return type == ROW_ALL;
-}
-
 /**
  * empathy_account_chooser_dup_account:
  * @chooser: an #EmpathyAccountChooser
@@ -490,7 +498,7 @@ empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 
 		gtk_list_store_prepend (store, &iter);
 		gtk_list_store_set (store, &iter,
-				    COL_ACCOUNT_TEXT, _("All accounts"),
+				    COL_ACCOUNT_TEXT, _("All"),
 				    COL_ACCOUNT_ENABLED, TRUE,
 				    COL_ACCOUNT_POINTER, NULL,
 				    COL_ACCOUNT_ROW_TYPE, ROW_ALL,
@@ -789,8 +797,7 @@ account_chooser_filter_ready_cb (gboolean is_enabled,
 	}
 
 	g_object_unref (account);
-	g_free (iter);
-	g_slice_free (FilterResultCallbackData, fr_data);
+	filter_result_callback_data_free (fr_data);
 }
 
 static void
@@ -816,18 +823,13 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 	if (account == NULL)
 		return;
 
-	data = g_slice_new0 (FilterResultCallbackData);
-	data->chooser = chooser;
-	data->account = g_object_ref (account);
-	data->iter = g_memdup (iter, sizeof (GtkTreeIter));
+	data = filter_result_callback_data_new (chooser, account, iter);
 
 	if (priv->filter)
 		priv->filter (account, account_chooser_filter_ready_cb,
 			      (gpointer) data, priv->filter_data);
 	else
 		account_chooser_filter_ready_cb (TRUE, (gpointer) data);
-
-	g_object_unref (account);
 }
 
 static void
@@ -963,6 +965,77 @@ empathy_account_chooser_filter_is_connected (
 	callback (is_connected, callback_data);
 }
 
+typedef struct {
+	EmpathyAccountChooserFilterResultCallback callback;
+	gpointer                                  user_data;
+} FilterCallbackData;
+
+static void
+conn_prepared_cb (GObject *conn,
+		GAsyncResult *result,
+		gpointer user_data)
+{
+	FilterCallbackData *data = user_data;
+	GError             *myerr = NULL;
+	TpCapabilities     *caps;
+
+	if (!tp_proxy_prepare_finish (conn, result, &myerr)) {
+		data->callback (FALSE, data->user_data);
+		g_slice_free (FilterCallbackData, data);
+		return;
+	}
+
+	caps = tp_connection_get_capabilities (TP_CONNECTION (conn));
+	data->callback (tp_capabilities_supports_text_chatrooms (caps),
+			data->user_data);
+
+	g_slice_free (FilterCallbackData, data);
+}
+
+/**
+ * empathy_account_chooser_filter_supports_multichat:
+ * @account: a #TpAccount
+ * @callback: an #EmpathyAccountChooserFilterResultCallback accepting the result
+ * @callback_data: data passed to the @callback
+ * @user_data: user data or %NULL
+ *
+ * An #EmpathyAccountChooserFilterFunc that returns accounts that both
+ * support multiuser text chat and are connected.
+ *
+ * Returns (via the callback) TRUE if @account both supports muc and is connected
+ */
+void
+empathy_account_chooser_filter_supports_chatrooms (
+	TpAccount                                 *account,
+	EmpathyAccountChooserFilterResultCallback  callback,
+	gpointer                                   callback_data,
+	gpointer                                   user_data)
+{
+	TpConnection       *connection;
+	FilterCallbackData *cb_data;
+	GQuark              features[] = { TP_CONNECTION_FEATURE_CAPABILITIES, 0 };
+
+	if (tp_account_get_connection_status (account, NULL) !=
+		TP_CONNECTION_STATUS_CONNECTED) {
+		callback (FALSE, callback_data);
+		return;
+	}
+
+	/* check if CM supports multiuser text chat */
+	connection = tp_account_get_connection (account);
+	if (connection == NULL) {
+		callback (FALSE, callback_data);
+		return;
+	}
+
+	cb_data = g_slice_new0 (FilterCallbackData);
+	cb_data->callback = callback;
+	cb_data->user_data = callback_data;
+
+	tp_proxy_prepare_async (connection, features, conn_prepared_cb,
+		cb_data);
+}
+
 gboolean
 empathy_account_chooser_is_ready (EmpathyAccountChooser *self)
 {
