@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2002-2007 Imendio AB
- * Copyright (C) 2007-2008 Collabora Ltd.
+ * Copyright (C) 2007-2010 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -42,25 +42,25 @@
 #include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-marshal.h>
 
 #include "empathy-chat.h"
 #include "empathy-spell.h"
 #include "empathy-contact-list-store.h"
 #include "empathy-contact-list-view.h"
 #include "empathy-contact-menu.h"
+#include "empathy-input-text-view.h"
 #include "empathy-search-bar.h"
 #include "empathy-theme-manager.h"
 #include "empathy-smiley-manager.h"
 #include "empathy-ui-utils.h"
 #include "empathy-string-parser.h"
+#include "extensions/extensions.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
 
-#define CHAT_DIR_CREATE_MODE  (S_IRUSR | S_IWUSR | S_IXUSR)
-#define CHAT_FILE_CREATE_MODE (S_IRUSR | S_IWUSR)
 #define IS_ENTER(v) (v == GDK_KEY_Return || v == GDK_KEY_ISO_Enter || v == GDK_KEY_KP_Enter)
-#define MAX_INPUT_HEIGHT 150
 #define COMPOSING_STOP_TIMEOUT 5
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChat)
@@ -87,7 +87,6 @@ struct _EmpathyChatPriv {
 	TpHandleType       handle_type;
 	gint               contacts_width;
 	gboolean           has_input_vscroll;
-	gint               topic_width;
 
 	/* TRUE if spell checking is enabled, FALSE otherwise.
 	 * This is to keep track of the last state of spell checking
@@ -101,6 +100,8 @@ struct _EmpathyChatPriv {
 
 	/* Source func ID for update_misspelled_words () */
 	guint              update_misspelled_words_id;
+	/* Source func ID for save_paned_pos_timeout () */
+	guint              save_paned_pos_id;
 
 	GtkWidget         *widget;
 	GtkWidget         *hpaned;
@@ -172,6 +173,8 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (EmpathyChat, empathy_chat, GTK_TYPE_BIN);
 
+static gboolean update_misspelled_words (gpointer data);
+
 static void
 chat_get_property (GObject    *object,
 		   guint       param_id,
@@ -661,6 +664,17 @@ OUT:
 }
 #endif
 
+static gboolean
+nick_command_supported (EmpathyChat *chat)
+{
+	EmpathyChatPriv * priv = GET_PRIV (chat);
+	TpConnection    *connection;
+
+	connection = empathy_tp_chat_get_connection (priv->tp_chat);
+	return tp_proxy_has_interface_by_id (connection,
+		EMP_IFACE_QUARK_CONNECTION_INTERFACE_RENAMING);
+}
+
 static void
 chat_command_clear (EmpathyChat *chat,
 		    GStrv        strv)
@@ -760,24 +774,28 @@ chat_command_msg (EmpathyChat *chat,
 }
 #endif
 
+static void
+callback_for_request_rename(TpProxy *proxy,
+		  const GError *error,
+		  gpointer user_data,
+		  GObject *weak_object)
+{
+	if (error != NULL) {
+		DEBUG ("Call to RequestRename method failed: %s",error->message);
+	}
+}
+
 static void
 chat_command_nick (EmpathyChat *chat,
 		   GStrv        strv)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	TpConnection *connection;
-	GHashTable *new_alias;
-	TpHandle handle;
+	TpProxy *proxy;
 
-	connection = tp_account_get_connection (priv->account);
-	handle = tp_connection_get_self_handle (connection);
-	new_alias = g_hash_table_new (g_direct_hash, g_direct_equal);
-	g_hash_table_insert (new_alias, GUINT_TO_POINTER (handle), strv[1]);
+	proxy = TP_PROXY (tp_account_get_connection (priv->account));
 
-	tp_cli_connection_interface_aliasing_call_set_aliases (connection, -1,
-		new_alias, NULL, NULL, NULL, NULL);
-
-	g_hash_table_destroy (new_alias);
+	emp_cli_connection_interface_renaming_call_request_rename (proxy, -1,
+		strv[1], callback_for_request_rename, NULL, NULL, NULL);
 }
 
 static void
@@ -814,43 +832,44 @@ typedef struct {
 	guint min_parts;
 	guint max_parts;
 	ChatCommandFunc func;
+	gboolean (*is_supported)(EmpathyChat *chat);
 	const gchar *help;
 } ChatCommandItem;
 
 static ChatCommandItem commands[] = {
-	{"clear", 1, 1, chat_command_clear,
+	{"clear", 1, 1, chat_command_clear, NULL,
 	 N_("/clear: clear all messages from the current conversation")},
 
-	{"topic", 2, 2, chat_command_topic,
+	{"topic", 2, 2, chat_command_topic, NULL,
 	 N_("/topic <topic>: set the topic of the current conversation")},
 
-	{"join", 2, 2, chat_command_join,
+	{"join", 2, 2, chat_command_join, NULL,
 	 N_("/join <chat room ID>: join a new chat room")},
 
-	{"j", 2, 2, chat_command_join,
+	{"j", 2, 2, chat_command_join, NULL,
 	 N_("/j <chat room ID>: join a new chat room")},
 
 #if 0
 /* FIXME: https://bugzilla.gnome.org/show_bug.cgi?id=623682 */
-	{"query", 2, 3, chat_command_query,
+	{"query", 2, 3, chat_command_query, NULL,
 	 N_("/query <contact ID> [<message>]: open a private chat")},
 
-	{"msg", 3, 3, chat_command_msg,
+	{"msg", 3, 3, chat_command_msg, NULL,
 	 N_("/msg <contact ID> <message>: open a private chat")},
 #endif
 
-	{"nick", 2, 2, chat_command_nick,
+	{"nick", 2, 2, chat_command_nick, nick_command_supported,
 	 N_("/nick <nickname>: change your nickname on the current server")},
 
-	{"me", 2, 2, chat_command_me,
+	{"me", 2, 2, chat_command_me, NULL,
 	 N_("/me <message>: send an ACTION message to the current conversation")},
 
-	{"say", 2, 2, chat_command_say,
+	{"say", 2, 2, chat_command_say, NULL,
 	 N_("/say <message>: send <message> to the current conversation. "
 	    "This is used to send a message starting with a '/'. For example: "
 	    "\"/say /join is used to join a new chat room\"")},
 
-	{"help", 1, 2, chat_command_help,
+	{"help", 1, 2, chat_command_help, NULL,
 	 N_("/help [<command>]: show all supported commands. "
 	    "If <command> is defined, show its usage.")},
 };
@@ -870,20 +889,33 @@ static void
 chat_command_help (EmpathyChat *chat,
 		   GStrv        strv)
 {
+	EmpathyChatPriv *priv;
 	guint i;
 
+	priv = GET_PRIV (chat);
+
 	/* If <command> part is not defined,
 	 * strv[1] will be the terminal NULL */
 	if (strv[1] == NULL) {
 		for (i = 0; i < G_N_ELEMENTS (commands); i++) {
-			empathy_chat_view_append_event (chat->view,
-				_(commands[i].help));
+			if (commands[i].is_supported != NULL) {
+				if (!commands[i].is_supported (chat)) {
+					continue;
+				}
+			}
+		    empathy_chat_view_append_event (chat->view,
+			    _(commands[i].help));
 		}
 		return;
 	}
 
 	for (i = 0; i < G_N_ELEMENTS (commands); i++) {
 		if (g_ascii_strcasecmp (strv[1], commands[i].prefix) == 0) {
+			if (commands[i].is_supported != NULL) {
+				if (!commands[i].is_supported (chat)) {
+					break;
+				}
+			}
 			chat_command_show_help (chat, &commands[i]);
 			return;
 		}
@@ -979,6 +1011,11 @@ chat_send (EmpathyChat  *chat,
 			if (c != '\0' && !g_ascii_isspace (c)) {
 				continue;
 			}
+			if (commands[i].is_supported != NULL) {
+				if (!commands[i].is_supported (chat)) {
+					continue;
+				}
+			}
 
 			/* We can't use g_strsplit here because it does
 			 * not deal correctly if we have more than one space
@@ -1108,7 +1145,9 @@ chat_state_changed_cb (EmpathyTpChat      *tp_chat,
 }
 
 static void
-chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
+chat_message_received (EmpathyChat *chat,
+	EmpathyMessage *message,
+	gboolean pending)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	EmpathyContact  *sender;
@@ -1127,7 +1166,7 @@ chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 			       chat);
 
 	priv->unread_messages++;
-	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message);
+	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message, pending);
 }
 
 static void
@@ -1135,7 +1174,7 @@ chat_message_received_cb (EmpathyTpChat  *tp_chat,
 			  EmpathyMessage *message,
 			  EmpathyChat    *chat)
 {
-	chat_message_received (chat, message);
+	chat_message_received (chat, message, FALSE);
 }
 
 static void
@@ -1176,48 +1215,6 @@ chat_send_error_cb (EmpathyTpChat          *tp_chat,
 	g_free (str);
 }
 
-/* WARNING: EXPLICIT CONTENT, keep away childrens!
- *
- * When a GtkLabel is set to wrap, it assume and hardcoded width. To change
- * that width we have to set a size request on the label... but that's not
- * possible because we want the window to be able to shrink, so we MUST request
- * width of 1. Note that the height of a wrapping label depends on its width.
- *
- * To work around that, here is what happens:
- * 1) size-request is first called, an hardcoded small width is requested by
- *    GtkLabel, which means also a too big height. We do nothing.
- * 2) size-allocate is called with the full width available, that's the width
- *    we really want to make wrap the label. We save that width and restart a
- *    size-request/size-allocate round.
- * 3) size-request is called a 2nd time, now we can tell the pango layout its
- *    width (we can't do that in step 2 because GtkLabel::size-request recreate
- *    the layout each time). When the layout has its width, we can know the
- *    height of the label and set its requisition. The width request is set to
- *    1px to make sure the window can shrink, the layout will fill all the
- *    available width anyway.
- */
-static void
-chat_topic_label_size_request_cb (GtkLabel *label,
-				   GtkRequisition *requisition,
-				   EmpathyChat *chat)
-{
-	EmpathyChatPriv *priv = GET_PRIV (chat);
-
-	if (gtk_label_get_line_wrap (label) && priv->topic_width > 0) {
-		PangoLayout *layout;
-		PangoRectangle rect;
-		gint ypad;
-
-		layout = gtk_label_get_layout (label);
-		pango_layout_set_width (layout, priv->topic_width * PANGO_SCALE);
-		pango_layout_get_extents (layout, NULL, &rect);
-		gtk_misc_get_padding (GTK_MISC (label), NULL, &ypad);
-
-		requisition->width = 1;
-		requisition->height = PANGO_PIXELS (rect.height) + ypad * 2;
-	}
-}
-
 static void
 chat_topic_label_size_allocate_cb (GtkLabel *label,
 				   GtkAllocation *allocation,
@@ -1226,8 +1223,6 @@ chat_topic_label_size_allocate_cb (GtkLabel *label,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 
 	if (!gtk_label_get_line_wrap (label)) {
-		priv->topic_width = -1;
-
 		if (pango_layout_is_ellipsized (gtk_label_get_layout (label)))
 			gtk_widget_show (priv->expander_topic);
 		else
@@ -1235,11 +1230,6 @@ chat_topic_label_size_allocate_cb (GtkLabel *label,
 
 		return;
 	}
-
-	if (priv->topic_width != allocation->width) {
-		priv->topic_width = allocation->width;
-		gtk_widget_queue_resize (GTK_WIDGET (label));
-	}
 }
 
 static void
@@ -1657,40 +1647,6 @@ chat_text_view_focus_in_event_cb (GtkWidget  *widget,
 	return TRUE;
 }
 
-static gboolean
-chat_input_set_size_request_idle (gpointer sw)
-{
-	gtk_widget_set_size_request (sw, -1, MAX_INPUT_HEIGHT);
-
-	return FALSE;
-}
-
-static void
-chat_input_size_request_cb (GtkWidget      *widget,
-			    GtkRequisition *requisition,
-			    EmpathyChat    *chat)
-{
-	EmpathyChatPriv *priv = GET_PRIV (chat);
-	GtkWidget       *sw;
-
-	sw = gtk_widget_get_parent (widget);
-	if (requisition->height >= MAX_INPUT_HEIGHT && !priv->has_input_vscroll) {
-		g_idle_add (chat_input_set_size_request_idle, sw);
-		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-						GTK_POLICY_NEVER,
-						GTK_POLICY_ALWAYS);
-		priv->has_input_vscroll = TRUE;
-	}
-
-	if (requisition->height < MAX_INPUT_HEIGHT && priv->has_input_vscroll) {
-		gtk_widget_set_size_request (sw, -1, -1);
-		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-						GTK_POLICY_NEVER,
-						GTK_POLICY_NEVER);
-		priv->has_input_vscroll = FALSE;
-	}
-}
-
 static void
 chat_input_realize_cb (GtkWidget   *widget,
 		       EmpathyChat *chat)
@@ -1838,6 +1794,123 @@ chat_spelling_build_menu (EmpathyChatSpell *chat_spell)
 	return menu;
 }
 
+typedef struct {
+	EmpathyChat  *chat;
+	gchar        *word;
+	gchar        *code;
+} EmpathyChatWord;
+
+static EmpathyChatWord *
+chat_word_new (EmpathyChat  *chat,
+		const gchar  *word,
+		const gchar  *code)
+{
+	EmpathyChatWord *chat_word;
+
+	chat_word = g_slice_new0 (EmpathyChatWord);
+
+	chat_word->chat = g_object_ref (chat);
+	chat_word->word = g_strdup (word);
+	chat_word->code = g_strdup (code);
+
+	return chat_word;
+}
+
+static void
+chat_word_free (EmpathyChatWord *chat_word)
+{
+	g_object_unref (chat_word->chat);
+	g_free (chat_word->word);
+	g_free (chat_word->code);
+	g_slice_free (EmpathyChatWord, chat_word);
+}
+
+static void
+chat_add_to_dictionary_activate_cb (GtkMenuItem     *menu_item,
+				    EmpathyChatWord *chat_word)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat_word->chat);
+
+	empathy_spell_add_to_dictionary (chat_word->code,
+					 chat_word->word);
+	priv->update_misspelled_words_id =
+		g_idle_add (update_misspelled_words, chat_word->chat);
+}
+
+static GtkWidget *
+chat_spelling_build_add_to_dictionary_item (EmpathyChatSpell *chat_spell)
+{
+	GtkWidget       *menu, *item, *lang_item, *image;
+	GList           *codes, *l;
+	gchar           *label;
+	const gchar     *code, *name;
+	EmpathyChatWord *chat_word;
+
+	codes = empathy_spell_get_enabled_language_codes ();
+	g_assert (codes != NULL);
+	if (g_list_length (codes) > 1) {
+		/* translators: %s is the selected word */
+		label = g_strdup_printf (_("Add '%s' to Dictionary"),
+					chat_spell->word);
+		item = gtk_image_menu_item_new_with_mnemonic (label);
+		g_free (label);
+		image = gtk_image_new_from_icon_name (GTK_STOCK_ADD,
+						      GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+					       image);
+
+		menu = gtk_menu_new ();
+
+		for (l = codes; l; l = l->next) {
+			code = l->data;
+			name = empathy_spell_get_language_name (code);
+			if (name == NULL)
+				continue;
+
+			lang_item = gtk_image_menu_item_new_with_label (name);
+
+			chat_word= chat_word_new (chat_spell->chat,
+						  chat_spell->word, code);
+			g_object_set_data_full (G_OBJECT (lang_item),
+				"chat-word", chat_word,
+				(GDestroyNotify) chat_word_free);
+
+			g_signal_connect (G_OBJECT (lang_item), "activate",
+				G_CALLBACK (chat_add_to_dictionary_activate_cb),
+				chat_word);
+			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), lang_item);
+		}
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
+	} else {
+		code = codes->data;
+		name = empathy_spell_get_language_name (code);
+		g_assert (name != NULL);
+		/* translators: first %s is the selected word,
+		 * second %s is the language name of the target dictionary */
+		label = g_strdup_printf (_("Add '%s' to %s Dictionary"),
+					chat_spell->word, name);
+		item = gtk_image_menu_item_new_with_mnemonic (label);
+		g_free (label);
+		image = gtk_image_new_from_icon_name (GTK_STOCK_ADD,
+						      GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+		chat_word = chat_word_new (chat_spell->chat, chat_spell->word,
+					   code);
+		g_object_set_data_full (G_OBJECT (item), "chat-word", chat_word,
+					(GDestroyNotify) chat_word_free);
+
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (chat_add_to_dictionary_activate_cb),
+				  chat_word);
+	}
+	g_list_free (codes);
+
+	gtk_widget_show_all (item);
+
+	return item;
+}
+
 static void
 chat_text_send_cb (GtkMenuItem *menuitem,
 		   EmpathyChat *chat)
@@ -1860,6 +1933,7 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	gchar                *str = NULL;
 	EmpathyChatSpell     *chat_spell;
 	GtkWidget            *spell_menu;
+	GtkWidget            *spell_item;
 	EmpathySmileyManager *smiley_manager;
 	GtkWidget            *smiley_menu;
 	GtkWidget            *image;
@@ -1917,13 +1991,14 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	if (!EMP_STR_EMPTY (str)) {
 		chat_spell = chat_spell_new (chat, str, start, end);
 		g_object_set_data_full (G_OBJECT (menu),
-					"chat_spell", chat_spell,
+					"chat-spell", chat_spell,
 					(GDestroyNotify) chat_spell_free);
 
 		item = gtk_separator_menu_item_new ();
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 
+		/* Spelling suggestions */
 		item = gtk_image_menu_item_new_with_mnemonic (_("_Spelling Suggestions"));
 		image = gtk_image_new_from_icon_name (GTK_STOCK_SPELL_CHECK,
 						      GTK_ICON_SIZE_MENU);
@@ -1932,6 +2007,17 @@ chat_input_populate_popup_cb (GtkTextView *view,
 		spell_menu = chat_spelling_build_menu (chat_spell);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), spell_menu);
 
+
+		spell_item = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (spell_menu), spell_item);
+		gtk_widget_show (spell_item);
+
+		/* Add to dictionary */
+		spell_item = chat_spelling_build_add_to_dictionary_item (chat_spell);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (spell_menu), spell_item);
+		gtk_widget_show (spell_item);
+
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 	}
@@ -1955,6 +2041,7 @@ chat_log_filter (TplEntry *log,
 
 	for (; pending; pending = g_list_next (pending)) {
 		if (empathy_message_equal (message, pending->data)) {
+			g_object_unref (message);
 			return FALSE;
 		}
 	}
@@ -1981,7 +2068,7 @@ show_pending_messages (EmpathyChat *chat) {
 
 	for (l = messages; l != NULL ; l = g_list_next (l)) {
 		EmpathyMessage *message = EMPATHY_MESSAGE (l->data);
-		chat_message_received (chat, message);
+		chat_message_received (chat, message, TRUE);
 	}
 }
 
@@ -2442,18 +2529,33 @@ conf_spell_checking_cb (GSettings *gsettings_chat,
 	priv->spell_checking_enabled = spell_checker;
 }
 
+static gboolean
+save_paned_pos_timeout (gpointer data)
+{
+	EmpathyChat *self = data;
+	gint hpaned_pos;
+
+	hpaned_pos = gtk_paned_get_position (GTK_PANED (self->priv->hpaned));
+
+	g_settings_set_int (self->priv->gsettings_ui,
+			    EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
+			    hpaned_pos);
+
+	return FALSE;
+}
+
 static gboolean
 chat_hpaned_pos_changed_cb (GtkWidget* hpaned,
 		GParamSpec *spec,
 		gpointer user_data)
 {
 	EmpathyChat *chat = EMPATHY_CHAT (user_data);
-	gint hpaned_pos;
 
-	hpaned_pos = gtk_paned_get_position (GTK_PANED(hpaned));
-	g_settings_set_int (chat->priv->gsettings_ui,
-			    EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
-			    hpaned_pos);
+	if (chat->priv->save_paned_pos_id != 0)
+		g_source_remove (chat->priv->save_paned_pos_id);
+
+	chat->priv->save_paned_pos_id = g_timeout_add_seconds (1,
+		save_paned_pos_timeout, chat);
 
 	return TRUE;
 }
@@ -2486,7 +2588,6 @@ chat_create_ui (EmpathyChat *chat)
 	empathy_builder_connect (gui, chat,
 		"expander_topic", "notify::expanded", chat_topic_expander_activate_cb,
 		"label_topic", "size-allocate", chat_topic_label_size_allocate_cb,
-		"label_topic", "size-request", chat_topic_label_size_request_cb,
 		NULL);
 
 	g_free (filename);
@@ -2507,20 +2608,11 @@ chat_create_ui (EmpathyChat *chat)
 	gtk_widget_show (GTK_WIDGET (chat->view));
 
 	/* Add input GtkTextView */
-	chat->input_text_view = g_object_new (GTK_TYPE_TEXT_VIEW,
-					      "pixels-above-lines", 2,
-					      "pixels-below-lines", 2,
-					      "pixels-inside-wrap", 1,
-					      "right-margin", 2,
-					      "left-margin", 2,
-					      "wrap-mode", GTK_WRAP_WORD_CHAR,
-					      NULL);
+	chat->input_text_view = empathy_input_text_view_new ();
+
 	g_signal_connect (chat->input_text_view, "key-press-event",
 			  G_CALLBACK (chat_input_key_press_event_cb),
 			  chat);
-	g_signal_connect (chat->input_text_view, "size-request",
-			  G_CALLBACK (chat_input_size_request_cb),
-			  chat);
 	g_signal_connect (chat->input_text_view, "realize",
 			  G_CALLBACK (chat_input_realize_cb),
 			  chat);
@@ -2618,6 +2710,9 @@ chat_finalize (GObject *object)
 	if (priv->update_misspelled_words_id != 0)
 		g_source_remove (priv->update_misspelled_words_id);
 
+	if (priv->save_paned_pos_id != 0)
+		g_source_remove (priv->save_paned_pos_id);
+
 	g_object_unref (priv->gsettings_chat);
 	g_object_unref (priv->gsettings_ui);
 
@@ -2773,9 +2868,9 @@ empathy_chat_class_init (EmpathyChatClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
+			      _empathy_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
-			      1, EMPATHY_TYPE_MESSAGE);
+			      2, EMPATHY_TYPE_MESSAGE, G_TYPE_BOOLEAN);
 
 	g_type_class_add_private (object_class, sizeof (EmpathyChatPriv));
 }
@@ -3029,7 +3124,6 @@ empathy_chat_set_tp_chat (EmpathyChat   *chat,
 			  EmpathyTpChat *tp_chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	TpConnection    *connection;
 	GPtrArray       *properties;
 
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
@@ -3045,7 +3139,6 @@ empathy_chat_set_tp_chat (EmpathyChat   *chat,
 	}
 
 	priv->tp_chat = g_object_ref (tp_chat);
-	connection = empathy_tp_chat_get_connection (priv->tp_chat);
 	priv->account = g_object_ref (empathy_tp_chat_get_account (priv->tp_chat));
 
 	g_signal_connect (tp_chat, "destroy",
@@ -3190,12 +3283,6 @@ empathy_chat_get_contact_menu (EmpathyChat *chat)
 						 EMPATHY_CONTACT_FEATURE_LOG |
 						 EMPATHY_CONTACT_FEATURE_INFO);
 	}
-	else if (priv->contact_list_view) {
-		EmpathyContactListView *view;
-
-		view = EMPATHY_CONTACT_LIST_VIEW (priv->contact_list_view);
-		menu = empathy_contact_list_view_get_contact_menu (view);
-	}
 
 	return menu;
 }
@@ -3253,6 +3340,31 @@ empathy_chat_copy (EmpathyChat *chat)
 
 		gtk_text_buffer_copy_clipboard (buffer, clipboard);
 	}
+	else {
+		gint start_offset;
+		gint end_offset;
+		EmpathyChatPriv *priv = GET_PRIV (chat);
+
+		if (gtk_label_get_selection_bounds (GTK_LABEL (priv->label_topic),
+							       &start_offset,
+							       &end_offset)) {
+			gchar *start;
+			gchar *end;
+			gchar *selection;
+			const gchar *topic;
+			GtkClipboard *clipboard;
+
+			topic = gtk_label_get_text (GTK_LABEL (priv->label_topic));
+			start = g_utf8_offset_to_pointer (topic, start_offset);
+			end = g_utf8_offset_to_pointer (topic, end_offset);
+			selection = g_strndup (start, end - start);
+
+			clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+			gtk_clipboard_set_text (clipboard, selection, -1);
+
+			g_free (selection);
+		}
+	}
 }
 
 void
@@ -3350,3 +3462,10 @@ empathy_chat_messages_read (EmpathyChat *self)
 	}
 	priv->unread_messages = 0;
 }
+
+/* Return TRUE if on of the contacts in this chat is composing */
+gboolean
+empathy_chat_is_composing (EmpathyChat *chat)
+{
+  return chat->priv->compositors != NULL;
+}
